#!/usr/bin/env python3

"""gofile-put.py: gofile uploader"""
__version__ = "1.4"
__author__ = "ed <irc.rizon.net>"
__url__ = "https://ocv.me/dev/?gofile-put.py"
__credits__ = ["stackoverflow.com"]
__license__ = "MIT"
__copyright__ = 2021


# set default values here;
# runtime arguments like --token will override these
TOKEN = ""
FOLDER = ""
DESC = ""
PASSWORD = ""
TAGS = ""
LINKS_FILE = "gofile-links.txt"

# debug: log all requests
# NOTE: log will contain your api token
LOG_REQ = False


import os
import sys
import time
import json
import logging
import hashlib
import argparse
import threading
import http.client
from datetime import datetime


def eprint(*a, **ka):
    ka["file"] = sys.stderr
    print(*a, **ka)


class TFile(object):
    def __init__(self, *a, **ka):
        self.f = open(*a, **ka)
        self.nbw = 0
        for fun in "seek tell read close".split():
            setattr(self, fun, getattr(self.f, fun))

    def noop(self, *a, **ka):
        pass

    def write(self, buf):
        self.f.write(buf)
        self.nbw += len(buf)
        if self.nbw > 1024 * 1024:
            self.write = self.noop


logsink = None
if LOG_REQ:
    logsink = TFile("gofile-put.reqlog", "w", encoding="utf-8", errors="replace")
    logging.basicConfig()
    logging.getLogger().setLevel(logging.DEBUG)
    logging.root.handlers = [logging.StreamHandler(logsink)]
    rl = logging.getLogger("requests.packages.urllib3")
    rl.setLevel(logging.DEBUG)
    rl.propagate = True

    def hclog(*a):
        rl.log(logging.DEBUG, " ".join([str(x) for x in a]))

    http.client.print = hclog
    http.client.HTTPConnection.debuglevel = 1


try:
    import requests
    from requests_toolbelt import MultipartEncoder
except:
    eprint(f"please run this:  {sys.executable} -m pip install requests-toolbelt")
    sys.exit(1)


class RiceFormatter(argparse.HelpFormatter):
    def _get_help_string(self, action):
        suf = "\033[36m (default: \033[35m%(default)s\033[36m)\033[0m"
        if action.default is not argparse.SUPPRESS:
            defaulting_nargs = [argparse.OPTIONAL, argparse.ZERO_OR_MORE]
            if action.option_strings or action.nargs in defaulting_nargs:
                return action.help + suf
        return action.help


class FileProgress(object):
    """proxies a file object and tracks how much is read"""

    def __init__(self, f):
        self.f = f
        self._t0 = time.time()
        self._nread = 0

        f.seek(0, 2)
        self._size = f.tell()
        self.len = self._size  # requests_toolbelt:total_len optimization
        f.seek(0)

        for op in ["seek", "tell", "fileno"]:
            setattr(self, op, getattr(f, op))

    def read(self, *a, **ka):
        ret = self.f.read(*a, **ka)
        self._nread += len(ret)
        return ret


class ProgressPrinter(object):
    """prints the progress of a file read"""

    def __init__(self):
        self.f = None

        t = threading.Thread(target=self.worker)
        t.daemon = True
        t.start()

    def worker(self):
        while True:
            time.sleep(0.05)
            f = self.f
            if not f:
                continue

            mb_read = f._nread / 1024 / 1024
            mb_size = f._size / 1024 / 1024

            percent = mb_read * 100.0 / mb_size
            td = time.time() - f._t0
            speed = mb_read / td

            eprint(
                f"\033[A  {percent:.2f}%, {speed:.2f} MiB/s ({mb_read:.2f} of {mb_size:.2f} MiB in {td:.2f} seconds)\n",
                end="",
            )


pp = ProgressPrinter()


def get_url():
    """gets an upload url using the gofile api"""
    r = requests.get("https://api.gofile.io/getServer")
    server = r.json()["data"]["server"]
    return f"https://{server}.gofile.io/uploadFile"


def put_file(url, formname, filepath, data):
    """uploads a file to the given url"""
    filename = os.path.basename(filepath)
    with open(filepath, "rb", 128 * 1024) as f:
        pp.f = FileProgress(f)

        data.append((formname, (filename, pp.f, "application/octet-stream")))
        m = MultipartEncoder(fields=data)

        headers = {"Content-Type": m.content_type}
        r = requests.post(url, data=m, headers=headers)
        pp.f = None
        return r.text


def main():
    os.system("")  # enables ansi escape sequences (console control) on windows

    # fmt: off
    ap = argparse.ArgumentParser(formatter_class=RiceFormatter)
    ap.add_argument("files", nargs="+")

    g = ap.add_argument_group('gofile options')
    g.add_argument("--token", metavar="BASE64", default=TOKEN or None, help="api/account token")
    g.add_argument("--folder", metavar="UUID", default=FOLDER or None, help="target folder id")
    g.add_argument("--desc", metavar="TXT", default=DESC or None, help="description")
    g.add_argument("--pwd", metavar="HUNTER2", default=PASSWORD or None, help="password")
    g.add_argument("--tags", metavar="T,T,T", default=TAGS or None, help="tags (comma-separated)")
    g.add_argument("--expire", metavar="UNIX_TS", default=0, type=int, help="expiration time (unix timestamp)")

    g = ap.add_argument_group('program options')
    g.add_argument("-lf", metavar="PATH", default=LINKS_FILE, help="write download links to PATH, disable with 'no' or blank")

    ar = ap.parse_args()
    # fmt: on

    ok = True
    for fpath in ar.files:
        if not os.path.exists(fpath):
            eprint(f"file not found: [{fpath}]")
            ok = False

    if not ok:
        sys.exit(1)

    logfile = None
    if ar.lf and ar.lf != "no":
        logfile = open(ar.lf, "a+", encoding="utf-8")

        now = time.time()
        now = datetime.utcfromtimestamp(now).strftime("%Y-%m-%d, %H:%M:%S.%f")[:-3]
        logfile.write(f"\n{now}\n")

    uploads = []
    slep = False
    for fpath in ar.files:
        if slep:
            # try to avoid {"status":"moveError","data":{}} from gofile
            time.sleep(3)

        slep = True
        url = get_url()
        eprint(f"\nuploading [{fpath}] to [{url}]\n\n", end="")

        if False:
            url = "http://127.1:3923/?pw=wark"
            field = "f"
            args = [("act", "bput")]
        else:
            field = "file"
            args = []
            for arg, field in [
                ("token", "token"),
                ("folder", "folderId"),
                ("desc", "description"),
                ("pwd", "password"),
                ("tags", "tags"),
                ("expire", "expire"),
            ]:
                v = getattr(ar, arg)
                if v:
                    args.append((field, v))

        txt = put_file(url, field, fpath, args)
        eprint("\033[36m" + str(txt) + "\033[0m")

        try:
            j = json.loads(txt)
            if j["status"] != "ok":
                raise Exception("gofile is not doing ok")
            link = j["data"]["downloadPage"]
            md5 = j["data"]["md5"]
        except:
            # eprint("  (failed to parse gofile response; continuing)")
            # continue
            raise

        uploads.append([fpath, md5, link])
        if logfile:
            logfile.write(f"{md5}  {link}  {fpath}\n")

    if logfile:
        logfile.close()

    if logsink:
        logsink.close()

    if not uploads:
        return

    eprint("\n\033[32mupload complete\033[0m")
    for fpath, md5, link in uploads:
        eprint(f"{md5}  {link}  {fpath}")

    eprint("\nverifying checksums (safe to abort with CTRL-C)\n")
    bad_uploads = []
    for fpath, md5, link in uploads:
        eprint(f"\033[Averifying [{fpath}]\033[K\n", end="")
        with open(fpath, "rb", 128 * 1024) as f:
            pp.f = FileProgress(f)
            hasher = hashlib.md5()
            while True:
                buf = pp.f.read(32 * 1024)
                if not buf:
                    break

                hasher.update(buf)
            pp.f = None

        digest = hasher.hexdigest()
        if digest.lower() != md5:
            eprint("BAD UPLOAD: [{digest}] != [{md5}] for [{fpath}]\n\n")
            bad_uploads.append(fpath)

    if bad_uploads:
        eprint("\033[AERROR: The following uploads are corrupted:\033[J")
        for fpath in bad_uploads:
            eprint(fpath)
    else:
        eprint("\033[Aall OK\033[J")


if __name__ == "__main__":
    try:
        main()
    except KeyboardInterrupt:
        eprint("OPYTHAT")
